/*
 * File:    pfodMAX31855.h
 * (c)2014 Forward Computing and Control Pty. Ltd
 * see http://www.pfod.com.au
 *
 *  This library is for the Adafruit Thermocouple Amplifier MAX31855 breakout board
 *  http://www.adafruit.com/products/269
 *   and type K therocouple
 *  http://www.adafruit.com/products/270
 *
 *  This library differs from the one provided by Adafruit in that
 * the library does not have any delays and returns the raw reading as an int.
 * This means the Arduino is not locked up waiting for a reading and the integer
 * raw reading can be efficiently averaged before converting to a float for display.
 * 
 * NOTE: No argument constructor uses DI_Pin = A5;  CLK_Pin = A3; CS_Pin = A4; 
 *
 * To start a measurement call 
 * triggerSensor() 
 *
 * triggerSensor can be called every loop as subsequent calls do nothing until a reading has been returned by 
 * getTypeKTemperature();
 * 
 * Calls to getTypeKTemperature() will return pfodMAX31855.NO_DATA (0) until a reading is available
 * When a reading is available getTypeKTemperature() will return either the raw reading (16*Kelvin)
 * or < pfodMAX31855.NO_DATA if error (pfodMAX31855.FAULT_OPEN_CIRCUIT (-1) or pfodMAX31855.FAULT_TO_VCC (-2) or pfodMAX31855.FAULT_TO_GND (-3) )
 *
 * Sample code
#include "pfodMAX31855.h"
#include <elapsedMillis.h>

pfodMAX31855 typeKTemp;
elapsedMillis readingTimer;

void setup() {
  Serial.begin(9600);
  for (int i=5; i>0; i--) {
    delay(1000);
    Serial.print(i);
    Serial.print(" ");
  }
  Serial.println();
  readingTimer = 0;
}

void loop() {
  if (readingTimer > 1000) {
    typeKTemp.triggerSensor();
    readingTimer -= 1000; // reset
  }
  int reading = typeKTemp.getTypeKTemperature();

  if (reading != typeKTemp.NO_DATA) { // have a reading
    Serial.print(millis()/1000.0); // output secs since restart
    Serial.print(F(",Sec,")); // csv comma separator
    if (reading < typeKTemp.NO_DATA) {
      Serial.println(reading);
      Serial.println(F(" ,Invalid Reading, ,Invalid Reading"));
    } else {
      Serial.print(typeKTemp.convertToFahrenheit(reading));
      Serial.print(F(",F,"));
      Serial.print(typeKTemp.convertToCelsius(reading));
      Serial.println(F(",C"));
    }
  }
}
*
 */

#ifndef pfodMAX31855_h
#define pfodMAX31855_h
#include "Arduino.h"
class microsTimer {
private:
  unsigned long us;
public:
  microsTimer(void) {
    us = micros();
  }
  microsTimer(unsigned long val) {
    us = micros() - val;
  }
  microsTimer(const microsTimer &orig) {
    us = orig.us;
  }
  operator unsigned long() const {
    return micros() - us;
  }
  microsTimer & operator =(const microsTimer &rhs) {
    us = rhs.us;
    return *this;
  }
  microsTimer & operator =(unsigned long val) {
    us = micros() - val;
    return *this;
  }
  microsTimer & operator -=(unsigned long val) {
    us += val;
    return *this;
  }
  microsTimer & operator +=(unsigned long val) {
    us -= val;
    return *this;
  }
  microsTimer operator -(int val) const {
    microsTimer r(*this);
    r.us += val;
    return r;
  }
  microsTimer operator -(unsigned int val) const {
    microsTimer r(*this);
    r.us += val;
    return r;
  }
  microsTimer operator -(long val) const {
    microsTimer r(*this);
    r.us += val;
    return r;
  }
  microsTimer operator -(unsigned long val) const {
    microsTimer r(*this);
    r.us += val;
    return r;
  }
  microsTimer operator +(int val) const {
    microsTimer r(*this);
    r.us -= val;
    return r;
  }
  microsTimer operator +(unsigned int val) const {
    microsTimer r(*this);
    r.us -= val;
    return r;
  }
  microsTimer operator +(long val) const {
    microsTimer r(*this);
    r.us -= val;
    return r;
  }
  microsTimer operator +(unsigned long val) const {
    microsTimer r(*this);
    r.us -= val;
    return r;
  }
};

class pfodMAX31855 {
public:
  /**
   * Use this for Uno type boards
   */
  pfodMAX31855(); //sets DI_Pin = A5; CLK_Pin = A3 CS_Pin = A4; 

  /**
   * This constructor uses 
   * DI_Pin, is the input data pin
   * CLK_Pin is the output clock pin to clock the Sensor's data output
   * CS_Pin is the chip select pin
   */
  pfodMAX31855(byte DI_Pin, byte CLK_Pin, byte CS_Pin);
  void triggerSensor(); // trigger sensor to start taking reading.
  int getTypeKTemperature(void); // returns NO_DATA (0) (absolute zero) if no reading or not enabled since last reading
  // return < NO_DATA (ie. FAULT_OPEN_CIRCUIT or FAULT_TO_VCC or FAULT_TO_GND if error)
  // returns > NO_DATA for valid reading
  float convertToFahrenheit(int reading);
  float convertToCelsius(int reading);
  static const int NO_DATA = 0;
  static const int FAULT_OPEN_CIRCUIT = -1;
  static const int FAULT_TO_VCC = -2;
  static const int FAULT_TO_GND = -3;
private:
  void initMAX31855(byte DI_Pin, byte CLK_Pin, byte CS_Pin);
  int processReading(uint32_t reading32);
  byte DIpin;
  byte CLKpin;
  byte CSpin;
  int read_result;
  // read states
  static const int kelvinOffset = 4370; //(=273.125 * 16) i.e 0.022 degC error;
  static const byte GETTING_DATA = 0;
  static const byte SELECT_TO_LOW = 1;
  static const byte DATA_FINISHED = 2;
  static const byte CLK_FINISHED = 3;
  static const byte START_READ = 4;
  byte readState;
  byte clockCounter;
  static const int clkDelay = 2;
  uint32_t data;
  microsTimer clkTimer;
  microsTimer startupTimer; // wait for MAX32155 to stabalize
  boolean started; // false until MAX31855 settled
};
#endif

